﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using System.Configuration;
using System.Security.Authentication;

/**
 * Sample code for the GetPickupRequestPrice Canada Post service.
 * 
 * The GetPickupRequestPrice service gets the price of a Pickup Request.
 * 
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * 
 */
namespace GetPickupRequestPrice
{
    class GetPickupRequestPrice
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_Pickup_DotNet_Samples\REST\pickup\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var customerNumber = ConfigurationSettings.AppSettings["customerNumber"];

            var url = "https://ct.soa-gw.canadapost.ca/enab/" + customerNumber + "/pickuprequest/price"; // REST URL

            var method = "POST"; // HTTP Method
            String responseAsString = ".NET Framework " + Environment.Version.ToString() + "\r\n\r\n";

            PickupDetailsType pickupDetails = new PickupDetailsType();
            pickupDetails.date = new DateTime(2019, 2, 15);
            
            try
            {
                // Serialize shipment object to String
                StringBuilder pickupRequestSb = new StringBuilder();
                XmlWriter pickupDetailsXml = XmlWriter.Create(pickupRequestSb);
                pickupDetailsXml.WriteProcessingInstruction("xml", "version=\"1.0\" encoding=\"UTF-8\"");
                XmlSerializer serializerRequest = new XmlSerializer(typeof(PickupDetailsType));
                serializerRequest.Serialize(pickupDetailsXml, pickupDetails);

                // Create REST Request
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
                request.Method = method;

                // Set Basic Authentication Header using username and password variables
                string auth = "Basic " + Convert.ToBase64String(Encoding.Default.GetBytes(username + ":" + password));
                request.Headers = new WebHeaderCollection();
                request.Headers.Add("Authorization", auth);

                // Write Post Data to Request
                UTF8Encoding encoding = new UTF8Encoding();
                byte[] buffer = encoding.GetBytes(pickupRequestSb.ToString());
                request.ContentLength = buffer.Length;
                request.Headers.Add("Accept-Language", "en-CA");
                request.Accept = "application/vnd.cpc.pickuprequest+xml";
                request.ContentType = "application/vnd.cpc.pickuprequest+xml";
                Stream PostData = request.GetRequestStream();
                PostData.Write(buffer, 0, buffer.Length);
                PostData.Close();

                // Execute REST Request
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                responseAsString += "HTTP Response Status: " + (int)response.StatusCode + "\r\n\r\n";

                // Deserialize response to pickupRequestInfo object
                XmlSerializer serializer = new XmlSerializer(typeof(PickupRequestPriceNoCCType));
                TextReader reader = new StreamReader(response.GetResponseStream());
                PickupRequestPriceNoCCType pickupRequestPrice = (PickupRequestPriceNoCCType)serializer.Deserialize(reader);

                // Retrieve values from pickupRequestPrice object
                responseAsString += "Due Amount: " + pickupRequestPrice.dueamount + "\r\n";
                responseAsString += "Pre-tax Amount: " + pickupRequestPrice.pretaxamount + "\r\n";
                responseAsString += "GST Amount: " + pickupRequestPrice.gstamount + "\r\n";
                responseAsString += "HST Amount: " + pickupRequestPrice.hstamount + "\r\n";
                responseAsString += "PST Amount: " + pickupRequestPrice.pstamount + "\r\n";
            }
            catch (WebException webEx)
            {
                HttpWebResponse response = (HttpWebResponse)webEx.Response;

                if (response != null)
                {
                    responseAsString += "HTTP  Response Status: " + webEx.Message + "\r\n";

                    // Retrieve errors from messages object
                    try
                    {

                        // Deserialize xml response to messages object
                        XmlSerializer serializer = new XmlSerializer(typeof(messages));
                        TextReader reader = new StreamReader(response.GetResponseStream());
                        messages myMessages = (messages)serializer.Deserialize(reader);


                        if (myMessages.message != null)
                        {
                            foreach (var item in myMessages.message)
                            {
                                responseAsString += "Error Code: " + item.code + "\r\n";
                                responseAsString += "Error Msg: " + item.description + "\r\n";
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        // Misc Exception
                        responseAsString += "ERROR: " + ex.Message;
                    }
                }
                else
                {
                    // Invalid Request
                    responseAsString += "ERROR: " + webEx.Message;
                }

            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message;
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);
        }
    }
}
